#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#Datos: Todas las clases relacionadas con los datos, y sus correspondientes metodos

#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import variables
import types

class Registro(list):
	"""Clase que guarda un registro"""
	def __init__(self,variables):
		""" variables es el array de variables al que esta asociado el registro"""
		list.__init__(self)
		self.variables=variables
	def __setitem__(self,indice,valor):
		""" 
			Redefinicion de []= de la lista
			Almacena en el valor convertido por la variable asociada
		"""
		list.__setitem__(self,indice,self.variables[indice].interno(valor))

	def __getitem__(self,indice):
		""" Redefinicion del [] de la lista. 
		Si nos pasan una ristra, comprobamos si se corresponde a alguna variable
		Si nos pasan un objeto, comprobamos si se corresponde a alguna variable
		Si nos pasan un entero, devolvemos el valor de esa posicion
		"""
		if indice.__class__==types.IntType:
			return list.__getitem__(self,indice)
		else:
			return self.__getitem__(self.variables.obtenerindice(indice))
	def append(self,valor):
		""" Redefinicion del append de lista
		Comprueba previamente si el valor esta ok
		"""
		list.append(self,self.variables[self.__len__()].interno(valor))
		
	def set(self,lista):
		"""Esta funcion establece los valores del registro"""
		#PENDIENTE... debe comprobar la correctitud.. estudiar que hacer
		if not len(self): #Caso lista vacia
			for valor in lista:
				self.append(valor)
		else :
			return "ERROR"

class Listaficheros(list):
	"""Contenedor de la lista de ficheros"""
	def append(self,elemento):
		if elemento in self:
			self.remove(elemento)
		self.insert(0,elemento)

class Arrayvar(list): 
	"""Lista que guarda las variables. Hereda de list"""
	def __init__(self):
		list.__init__(self)
	def obtenerindice(self,parametro):
		"""
		Obtiene el indice de un determinado tipo, 
		ya sea por el nombre o por la referencia al objeto en si
		"""
		
		if parametro.__class__==types.StringType:
			for iter in self:  
				if iter.nombre == parametro:
					return self.index(iter)
			return False
		else: #El parametro lo tomamos como una referencia
			if parametro in self:
				return self.index(parametro)
			return False
	def arr_defaults(self):
		"""Devuelve un array con todos los valores por defecto"""
		return [x.valorpordefecto for x in self]



class contenedorregvar:
	"""Contenedor de registros y variables"""
	def __init__(self):
		self.datos=[] #Un array para datos
		self.vars=Arrayvar() #Un array de variables 
	def __getitem__(self,indice): #Acceso por []
		"""Devuelve el item segun un indice numerico, un nombre, o un objeto variable
			Si es un indice, se entiende que se quiere acceder a un registro
			Si es un nombre o el objeto variable, se entiende que se quiere acceder a la columna
		"""
		if indice.__class__==types.IntType:
			return self.datos[indice]
		else:
			return self.get_col(indice)
			
	def __setitem__(self,indice,valor): 
		"""
		Modificacion por []
		Si lo que se pasa es una string o un objeto, se entiende que se quiere
		asignar de golpe una columna
		"""
		if indice.__class__==types.IntType:
			self.datos[indice]=valor
		else: 
			for registro in range(len(self.datos)):
				self.datos[registro][indice]=valor[registro]

	def get_dat(self,indice=-1):
		"""Devuelve un registro, por defecto el último"""
		return self.datos[indice]

	def get_var(self,indice=-1):
		"""Devuelve una variable, por defecto la última"""
		return self.vars[indice] 

	def set_var(self,indice,valor):
		"""Escribe la variable asociada al indice"""
		self.vars[indice]=valor

	def get_col(self,indice):
		"""Devuelve un array con todos los valores asociados a
		la variable representada por indice
		"""
		return [valor[indice] for valor in self.datos]
			
	def get_cel(self,x,y):
		"""Devuelve el valor de una sola celda"""
		return self.datos[x][y]

	def set_cel(self,x,y,valor):
		"""Cambia el valor de una sola celda""" #Deberia comprobar si la celda existe
		self.datos[x][y]=valor

	def num_dat(self):
		return len(self.datos)

	def num_var(self):
		return len(self.vars)

	def app_dat(self,dato=None):
		"""Añade un dato a los ya existentes"""
		registro=Registro(self.vars)
		if dato: #Nos pasan algo
			if isinstance(dato,Registro):
				self.datos.append(dato)
			elif dato.__class__==types.ListType:
				registro.set(dato)
				self.datos.append(registro)
		else: #Nos hacemos la lista nosotros con los valores por defecto
			registro.set(self.vars.arr_defaults())
			self.datos.append(registro)

	def app_var(self,nombre=None,tipo="Entero",titulo=None,valores={},ancho=0,decimales=0,valorpordefecto=0):  
		"""Añade una variable de tipo tipo al array de variables. Ademas rellena los datos con el valor por defecto"""
		if not nombre:
			nombre="VAR"+str(self.num_var())
		if not titulo:
			titulo="Variable"+str(self.num_var())
		if nombre.__class__ == types.StringType:
			if tipo=="Entero":
				self.vars.append(variables.Entero(nombre,titulo,valores,ancho,decimales,valorpordefecto)) #Cambiar los valores por defecto
		else:
			self.vars.append(nombre) # es un objeto variable enterio

		for i in range(self.num_dat()):
			self[i].append(valorpordefecto)
	def arr_tit(self):
		"""Devuelve una lista con todos los nombres de las variables"""
		return [tipo.nombre for tipo in self.vars]
	def arr_tit_bool(self):
		"""Devuelve una lista con todos los nombres de las variables Booleanas"""
		lista = [] 
		for tipo in self.vars:
			if tipo.tipo=="Booleano":
				lista.append(tipo.nombre)
		return lista
	def arr_var(self):
		"""Devuelve el array con todos los objetos de las variables"""
		return [tipo for tipo in self.vars]
	def mat_var(self,variable):
		"""A esta funcion se le pasa un objeto variable. Si esta existe en la lista de variables, se devuelve la posicion que ocupa (equivalente a index)"""
		return self.vars.obtenerindice(variable)
	def nom_var(self,variable):
		"""A esta funcion se le pasa un nombre. Si este existe en la lista de variables, se devuelve la ipropia variable"""
		return self.vars[self.vars.obtenerindice(variable)]



