#!/usr/bin/python
# -*- coding: latin-1 -*-

#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys
sys.path.append("./lib/")
sys.path.append("./dialog/")
sys.path.append("./dialog/ui")
sys.path.append("./dialog/vprincipal/ui/")
sys.path.append("./dialog/dcalculo/")
sys.path.append("./dialog/dconfig/")

from qt import *
from qttable import *
from uivprincipal import vprincipal
from dcalculo import DCalculo
from dconfig import DConfig
from dfiltro import DFiltro
from dcrevar import DCrevar
from dgraficos import DGraficos
from dbuscar import DBuscar
from dcasos import DCasos

import listas
import string
import greytableitem

class VPrincipal(vprincipal):
	def __init__(self,parent): 
		"""Inicializador de la ventana principal"""
		import datos
		vprincipal.__init__(self,None)
		self.__conexiones()
		self.parentXXX=parent

		#Bloque de inicialización de dialogos
		self.dcalculo = DCalculo(self,"Dialogo de calculo",0,)
		self.dfiltro = DFiltro(self,"dialogo filtrado",0,)
		self.dcrevar = DCrevar(self,"creacion variables",0,)
		self.dgraficos = DGraficos(self,"Graficos!",0,)
		self.dbuscar = DBuscar(self,"busqueda",0,)
		self.dconfig = DConfig(self,"Dialogo Config",0,)
		self.dcasos = DCasos(self,"Dialogo de casos",0,)

		#Bloque de variables de control (VARIABLES DE ESTADO)
		self.fichero = False  # Variable de control sobre si hay un fichero abierto. No sube a driza porque puede ser que otros dialogos abran otros ficheros
		self.modificado = False  # Variable de control sobre si el fichero abierto ha sido modificado

		#Variables enteras que almacenan el numero de casillas deseadas para las tablas como excedente
		self.nreg=10 
		self.nvar=10

		# Configuracion de las tablas
		self.__inittabladat()
		self.__inittablavar(self.nvar)

	

		
		
	def parent(self):
		"""Funcion chorra para que funcione self.parent()"""
		return self.parentXXX

	def __conexiones(self):
		"""Funcion llamada en el constructor que almacena todas las conexiones accion-funcion"""
		self.connect(self.fileExitAction,SIGNAL("activated()"),self.__salirprograma)
		self.connect(self.archivoSalirAction,SIGNAL("activated()"),self.__salirprograma)
		self.connect(self.fileOpenAction,SIGNAL("activated()"),self.__abrirfichero)
		self.connect(self.fileSaveAsAction,SIGNAL("activated()"),self.__guardarcomo)
		self.connect(self.fileSaveAction,SIGNAL("activated()"),self.__guardar)
		self.connect(self.analizarMediaAction,SIGNAL("activated()"),self.__dialogocalculo)
		self.connect(self.programaConfigurarAction,SIGNAL("activated()"),self.__configurar)
		self.connect(self.table1,SIGNAL("valueChanged(int,int)"),self.__actualizardat)
		self.connect(self.table2,SIGNAL("valueChanged(int,int)"),self.__actualizarvar)
		self.connect(self.datosFiltrarAction,SIGNAL("activated()"),self.__abrirdialogofiltro)
		self.connect(self.modificarCrear_nuevas_variablesAction,SIGNAL("activated()"),self.__dcrevar)
		self.connect(self.graficosBarrasAction,SIGNAL("activated()"),self.__dgraficos)
		self.connect(self.editFindAction,SIGNAL("activated()"),self.__dbuscar)
		self.connect(self.editCopyAction,SIGNAL("activated()"),self.copy)


	def __inittabladat(self):
		"""Inicializa la tabla de datos"""
		self.table1.setSelectionMode(QTable.MultiRow)
		self.table1.setNumCols(16) #No funciona porque el show asume que es la cantidad de datos + nreg
		self.table1.setNumRows(64)



	def __inittablavar(self,nvars):
		"""Inicializa la tabla de variables"""
		self.table2.setSelectionMode(QTable.MultiRow)
		self.table2.setNumRows(nvars)
		titulos=self.table2.horizontalHeader()
		titulos.setLabel (0,"Nombre")
		titulos.setLabel (1,"Tipo")
		titulos.setLabel (2,"Etiqueta")
		titulos.setLabel (3,"Valores")
		titulos.setLabel (4,"Ancho")
		titulos.setLabel (5,"Decimales")
		titulos.setLabel (6,"Valor por defecto")
		titulos.setLabel (7,"Descripción")
		self.table2.setNumCols(8)
		for fila in range(self.table2.numRows()):
			self.table2.setItem(fila,3,self.__mixtotableitem())
			self.table2.setItem(fila,1,self.__combotableitem())		


	#Funciones Publicas
	def __showtablavar(self,pos=None):  
		""" Representa la variable pos en la tabla de variables. Las propiedades de la variable las lee de los datos.
		Si no se indica posicion, se entiende que se quiere rellenar toda la tabla
		"""
		if pos:
			self.__showvar(pos)
			#Actualizacion de titulo en la tabla de datos
			self.__showtitulodat(pos)
		else:
			for n in range(self.parent().dato.num_var()):
				self.__showvar(n)
			self.__showtitulodat()


	def __showtabladat(self,pos=None):
		"""Muestra el dato en la posicion pos
		Lee de los datos, así que deben existir previamente
		Comprueba si esta filtrado o no para cambiar el color
		Si no se indica posicion, se entiende que se quiere rellenar toda la tabla
		"""
		if pos:  
			#Se ha indicado la posición
			if pos < self.parent().dato.num_dat(): #Se trata de un dato ya representado
				for i in range(self.parent().dato.num_var()):
					if self.parent().filtro:
						if self.parent().dato[self.parent().filtro][pos]:
							self.table1.setText(pos,i,str(self.parent().dato[pos][i]))
						else:
							self.table1.setItem(pos,i,greytableitem.GrisTableItem(self.table1,QTableItem.OnTyping,repr(self.parent().dato[pos][i]))) # 
					else:
						self.table1.setText(pos,i,str(self.parent().dato[pos][i]))
			else:
				print ("ERRRORRRRR") #excepcion
		else: 
			#No se ha indicado la posicion, se rellena todo
			if (self.table1.numCols() < (self.parent().dato.num_var() + self.nvar)): #¿Hay más celdas de las que necesito?
				self.table1.setNumCols(self.parent().dato.num_var()+self.nvar)
			if (self.table1.numRows() < (self.parent().dato.num_dat()+self.nreg)):
				self.table1.setNumRows(self.parent().dato.num_dat()+self.nreg)
			#Este bucle puede ser optimizado, sacando el filtro.. #PENDIENTE
			for x in range(self.parent().dato.num_dat()):
				for y in range(self.parent().dato.num_var()):
					if self.parent().filtro:
						if self.parent().dato[self.parent().filtro][x]:
							self.table1.setText(x,y,str(self.parent().dato[x][y]))
						else:
							self.table1.setItem(x,y,greytableitem.GrisTableItem(self.table1,QTableItem.OnTyping,repr(self.parent().dato[x][y]))) 
					else:
						self.table1.setText(x,y,str(self.parent().dato[x][y]))
			self.__showtitulodat()

	def __showtitulodat(self,pos=None):
		"""Actualiza los titulos de la tabla de datos segun las variables"""
		titulos=self.table1.horizontalHeader()
		if pos:
			titulos.setLabel(pos,self.parent().dato.get_var(pos).titulo)
		else:
			i=0
			for nombre in self.parent().dato.arr_tit():
				titulos.setLabel(i,nombre)
				i=i+1

	def __showvar(self,pos):
		"""Muestra una unica variable (fila) en la tabla segun el listado de variables"""
		if pos < self.parent().dato.num_var():
			self.table2.setItem(pos,1,self.__combotableitem())	
			self.table2.setText(pos,0,str(self.parent().dato.get_var(pos).nombre))
			self.table2.setText(pos,2,str(self.parent().dato.get_var(pos).titulo))
			self.table2.setText(pos,4,str(self.parent().dato.get_var(pos).ancho))
			self.table2.setText(pos,5,str(self.parent().dato.get_var(pos).decimales))
			self.table2.setText(pos,6,str(self.parent().dato.get_var(pos).valorpordefecto))
			self.table2.setText(pos,7,str(self.parent().dato.get_var(pos).descripcion))
			self.table2.item(pos,1).setCurrentItem(self.parent().dato.get_var(pos).tipo)
		else: 
			print("EEEEERRROOOORRRR") #excepcion!, se pide una variable que no existe
	
			

	#Funciones privadas
	def __combotableitem(self,indice=0):
		"""Devuelve un nuevo objeto tipo combotableitem"""
		lista=QStringList()
		import listas
		for tipo in listas.variables():
			lista.append(tipo)
		return QComboTableItem(self.table2,lista)
	
	def __mixtotableitem(self):
		"""Devuelve un objeto mixto"""
		import buttontableitem
		lista = buttontableitem.buttontableitem(self.table2)
		return lista


	#Slots o como se llame
	def __salirprograma(self):
		"""Funcion de salida del programa"""
		qApp.exit(0)

	def __abrirfichero(self):
		""" Cuando pulsamos abrir un archivo """
		filter=""
		for fmt in listas.extensiones_fichero():
			filter=filter+"%s files (*.%s);;" % (fmt,string.lower(fmt))
		fn = QFileDialog.getOpenFileName(QString.null,filter,self,None,"Dialogo abrir fichero","")
		filename = str(fn)
		if filename:
			self.parent().cargarfichero(filename)
	
	def __guardar(self):
		"""Funcion de guardado del fichero. Si el fichero no es conocido se llama a __guardarcomo"""
		if self.fichero:
			self.parent().guardarfichero()
		else:
			self.__guardarcomo()


	def __guardarcomo(self):
		"""Abre un diálogo pidiendo el nombre de archivo y guarda en dicho archivo"""
		fn = QFileDialog.getSaveFileName(QString.null,QString.null,self)
		filename = str(fn)
		if filename:
			self.parent().guardarfichero(filename)

	def __dialogocalculo(self):
		"""Muestra el dialogo de calculo"""
		self.dcalculo.show()

	def __configurar(self):
		"""Muestra el dialogo de configuración"""
		self.dconfig.show()

	def __actualizardat(self,x,y):
		"""Actualiza los datos del objeto dato a partir de un cambio en la tabla de datos"""
		import datos
		#Hay variables sin definir
		#Si estamos en una casilla que corresponde a una variable no creada
		if y>=self.parent().dato.num_var():
			for i in range(self.parent().dato.num_var(),y+1):
				self.parent().crearvariable()
				self.__showtablavar(i) # PENDIENTEdeberia mostrarlo todos

		#Si estamos en una casilla que corresponde a un registro no creado
		if x>=self.parent().dato.num_dat():
			#Actualizamos los valores intermedios(Si es que existe)
			
			for valorintermedio in range (self.parent().dato.num_dat(),x):  
				self.parent().dato.app_dat()
				self.__showtabladat(valorintermedio)

			#Actualizamos la fila que ha utilizado el usuario
			registroaintroducir=datos.Registro(self.parent().dato.vars) #Tenemos que introducir un nuevo registro


			#REPENSAR ESTA PARTE
			for tipodato in range(self.parent().dato.num_var()): #PENDIENTE.. llamar tipodato a eso no tiene sentido
				if tipodato==y:
					try:
						registroaintroducir.append(self.table1.text(x,y).latin1()) #Coge el dato que acaba de introducir
					except ValueError:
						rc = QMessageBox.warning(self,'Bienvenido', 'El dato no es válido')
				else:
					valordefecto=self.parent().dato.get_var(tipodato).valorpordefecto
					registroaintroducir.append(valordefecto) # Introduce el valor por defecto en el array
					self.table1.setText(x,tipodato,str(valordefecto)) # Introduce el valor en la tabla q se muestra
			self.parent().dato.app_dat(registroaintroducir) #Introducir la celda + resto de valores por defecto

		#El valor introducido sobreescribe uno que ya existe
		else:
			try:
				self.parent().dato.set_cel(x,y,self.table1.text(x,y).latin1())
			except ValueError:
				rc = QMessageBox.warning(self,'Bienvenido', 'El dato no es válido')
		
	def __actualizarvar(self,x,y):
		"""Funcion a que conecta con la introduccion de nuevos datos en la tabla de variables"""
		if x>=self.parent().dato.num_var(): #Es una nueva variable
			#Variables intermedias (creadas con los valores por defecto)
			for valoresintermedios in range (self.parent().dato.num_var(),x):  
				self.parent().dato.app_var()
				self.__showtablavar(valoresintermedios)

			#Ultima variable
			self.parent().dato.app_var()
			variable=self.parent().dato.get_var()
			variable[y]=self.table2.text(x,y).latin1()
			self.__showtablavar(x)
		else: #Trabajamos sobre una variable ya existente
			if y==1:   # El usuario quiere cambiar el tipo
				import variables
				textoencuestion=self.table2.text(x,y).latin1() 
				variableencuestion=self.parent().dato.get_var(x)
				if textoencuestion == "Dinero":
					nuevavariable=variables.Dinero()
				elif textoencuestion == "Entero":
					nuevavariable=variables.Entero()
				elif textoencuestion == "Real":
					nuevavariable=variables.Real()
				elif textoencuestion == "Booleano":
					nuevavariable=variables.Booleano()
				elif textoencuestion == "Fecha":
					nuevavariable=variables.Booleano()
				else:
					print "ERROR"
				nuevavariable.update(variableencuestion)
				self.parent().dato.set_var(x,nuevavariable)
				self.show()  # Solucion provisional, no debería hacer falta actualizarlo todo
			elif y==3: # Estamos en el caso de los casos, mejor no hacer nada ... de momento
				pass
			else:
				self.parent().dato.get_var(x)[y]=str(self.table2.text(x,y).latin1())
		#En todos los casos, actualizamos el titulo de la tabla de datos

	def show(self):
		"""Esta funcion actualiza el contenido de las rejillas segun los datos
		Reimplementa el show de la clase base
		"""
		#Tabla de datos
		#El margen extra sugerido por las variables nvar y ndat
		self.__showtablavar()
		self.__showtabladat()
		#Titulo de datos
		vprincipal.show(self)

	def __abrirdialogofiltro(self):
		"""Muestra el dialogo de filtrado"""
		self.dfiltro.show()

	def __dcrevar(self):
		"""Muestra el dialogo de creacion de variables"""
		self.dcrevar.show()

	def __dgraficos(self):
		"""Muestra el dialogo de graficos"""
		self.dgraficos.show()

	def __dbuscar(self):
		"""Muestra el dialogo de busqueda"""
		self.dbuscar.show()

	def __removeselection(self,tabla):
		""" Esta funcion borra la seleccion de la tabla"""
		for seleccion in range(tabla.numSelections()):
			tabla.removeSelection(seleccion)

	def copy(self):
		"""Esta funcion pega el contenido del clipboard en la tabla actual"""
		tablaactual=self.tabWidget2.currentPageIndex()
		cadena=""
		if tablaactual == 0:
			tabla=self.table1
		else:
			tabla=self.table2
		for seleccion in range(tabla.numSelections()):
			for fila in range(tabla.selection(seleccion).topRow(),tabla.selection(seleccion).bottomRow()+1):
				for columna in range(tabla.selection(seleccion).leftCol(),tabla.selection(seleccion).rightCol()+1):
					if tabla.item(fila,columna):
						cadena+=str(fila)+","+str(columna)+","+str(tabla.item(fila,columna).text())+" "
		cb = QApplication.clipboard()
		cb.setText(cadena)
		self.__removeselection(tabla)
#		Pendiente segun la seleccion
#		if cb.supportsSelection() :
#							cb.setText( self.__uri, QClipboard.Selection )

